from render import get_request

def set(key, value, expires='Expires=Mon, 19-Jan-2080 14:09:52 GMT;', path='/',request=None):
    str_cookie = "%s=%s"%(key,value)
    if path:
        str_cookie+=";Path=%s"%path
    if expires:
        str_cookie+=(";%s"%expires)
    if request is None:
        request = get_request()
    request.res.headers.add_header(
        'Set-Cookie',str_cookie
    )

def delete(key, path='/',request=None):
    if request is None:
        request = get_request()
    request.res.headers.add_header(
        'Set-Cookie',
        '%s=EXPIRED;Expires=Mon, 19-Jan-1980 14:09:52 GMT;Path=%s'%(
            key, path
        )
    )



#coding:utf-8
from os.path import abspath,dirname,join,normpath
import sys

#初始化python的查找路径
PREFIX = normpath(dirname(dirname(abspath(__file__))))
if PREFIX not in sys.path:
    sys.path=[PREFIX] + sys.path

#数据库配置
WTREE_HOST = "localhost:3306:wtree:root:111111"
DATABASE_CONFIG = None

#是否使用多线程
THREAD_SAFE = False

#是否启用在线调试
DEBUG = True

#是否使用优化过的css，js
ORG_CSS_JS = True

#是否启用Mako的自动检测模板更新
MAKO_FILESYSTEM_CHECK = True

#是否python由提供静态文件的服务
SERVER_STATIC_FILE = True

#静态文件服务器的域名
FILE_HOST = ""

#Javascript文件服务器的域名
JS_FILE_HOST = ""

#Memcache的地址
MEMCACHED_ADDR = None
#MEMCACHED_ADDR = ["127.0.0.1:11211"]

#是否禁用进程内缓存
DISABLE_LOCAL_CACHED = False

#系统发信人的邮箱地址和显示名称
SYS_EMAIL_SENDER = "stdyun@sina.com"
SYS_EMAIL_SENDER_NAME = "stdyun"

#发信的SMTP服务器，用户名，密码
SMTP = "smtp.sina.com"
SMTP_USERNAME = "stdyun"
SMTP_PASSWORD = "xxxxxx"

try:
    from local_config import *
except ImportError:
    print "WARNING : local_config not exist"

if not DATABASE_CONFIG:
    DATABASE_CONFIG = {
        "wtree": {
            "master": WTREE_HOST,
            "tables": (
                "*",
                "user","user_apply","user_password","user_email",
                "user_profile",
            ),
        },
    }




#coding:utf-8

from os.path import abspath,dirname,join,normpath
import sys

#初始化python的查找路径
PREFIX = normpath(dirname(dirname(dirname(abspath(__file__)))))
if PREFIX not in sys.path:
    sys.path=[PREFIX] + sys.path




from mypyconfig import LOOKUP,FUNC_MODULE_PREFIX_LEN,LOCAL
from types import FunctionType
from byteplay import Code, opmap

from decorator import decorator

from hmako.template import Template
import hmako.lookup


class Render(object): 
    __lookup = hmako.lookup.TemplateLookup(**LOOKUP)
    
    def __init__(self,template=None):
        self.__template = template
        
    def __getattr__(self,name): 
        return None

    def __call__(self,**kwds):
        return self.__lookup.get_template(self.__template).render(G=self,**kwds)


def get_request():
    return LOCAL.request

def _transmute(opcode, arg):
    if (
        (opcode == opmap['LOAD_GLOBAL']) and
        (arg == 'G' or arg == 'request')
    ):
        return opmap['LOAD_FAST'], arg
    return opcode, arg

def render_para_func(func):
    code = Code.from_code(func.func_code)
    code.args = tuple(['request','G'] + list(code.args))
    code.code = [_transmute(op, arg) for op, arg in code.code]
    func.func_code = code.to_code()
    return func


def render_func(func):
    template = "%s/%s.htm"%(
        func.__module__[FUNC_MODULE_PREFIX_LEN:].replace(".", "/"),
        func.__name__
    )
    def  _wrap_func(func,*args, **kwds):
        G=Render(template)
        request = get_request()
        r = func(request,G,*args, **kwds)

        if type(r) is str:
            return r
        elif r is False:
            return ''
        else:
            return G(request=request)

    f = decorator(_wrap_func, func)
    
    func = render_para_func(func)

    return f


def render_doc(func):
    template = Template(
        func.__doc__,
        input_encoding='utf-8',
        output_encoding='utf-8',
        disable_unicode=True,
        default_filters=['str', 'h']
    )

    def _func(func, *args, **kwds):
        request = get_request()
        G=Render()

        r = func(request,G,*args, **kwds)

        if type(r) is str:
            return r
        else:
            return template.render(G=G,request=request)

    f = decorator(_func, func)
    
    func = render_para_func(func)
    
    return f

if __name__ == "__main__":

    @render_doc
    def test():
        """
        my name is ${G.x}
        request is ${request}
        %if G.not_existxxxx:
        xxx
        %else:
            ${G.not_existxxxx}
        %endif
        """
        G.x="zsp<"

    print test()



#coding:utf-8


try:
    from myconf.config import ORG_CSS_JS
except:
    ORG_CSS_JS = False

try:
    from myconf.config import JS_FILE_HOST
except:
    JS_FILE_HOST = ""

if ORG_CSS_JS:

    my = "/js/my.js"

    reg = "/js/reg.js"

else:

    my = "%s/js/1tAg~my.js"%JS_FILE_HOST

    reg = "%s/js/1tAw~reg.js"%JS_FILE_HOST



import time
import struct
import zlib

def gzcompress(data):
    _compress = zlib.compressobj(5,zlib.DEFLATED,-zlib.MAX_WBITS,zlib.DEF_MEM_LEVEL,0)
    length = len(data)
    _size = length
    _crc = zlib.crc32(data)
    out = "\037\213\010\0%s\002\377%s%s%s%s"%(
        struct.pack('<L', long(time.time())),
        _compress.compress(data),
        _compress.flush(),
        struct.pack('<l', _crc),
        struct.pack('<L', _size & 0xffffffffL)
    )
    return out

def GzipMiddleware(func):
    def _func(request):
        result = func(request)
        if result:
            headers = request.res.headers
            result_len = len(result)
            if result_len>4096 and ('gzip' in request.environ.get('HTTP_ACCEPT_ENCODING', '')):
                result = gzcompress(result)
                result_len = len(result)
                headers.add_header("Content-Encoding","gzip")
                return result
            headers.add_header( 'Content-Length', str(result_len) )                
        return result
    return _func


#coding:utf-8

#是否启用在线调试
DEBUG = False

#是否启用Mako的自动检测模板更新
MAKO_FILESYSTEM_CHECK = False

#是否python由提供静态文件的服务
SERVER_STATIC_FILE = False

#是否使用原始的css，js
ORG_CSS_JS = True

#Javascript文件服务器的域名
JS_FILE_HOST = ""

#静态文件服务器的域名
FILE_HOST = ""

#数据库配置
WTREE_HOST = "localhost:3306:wtree:root:password"



#! /usr/bin/env python
#coding=utf-8
import re
EMAIL_VALID = re.compile(r"^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$")

def email_valid(email):
    return EMAIL_VALID.match(email)


WHITE_SPACE = "\r\n \tĦĦ"


"""HTTPServer extension to run WSGI applications. http://amix.dk/blog/viewEntry/19472"""
try:
    import cStringIO as StringIO
except:
    import StringIO

import sys
import urllib

import web
import httpserver
import ioloop


class WSGIServer(httpserver.HTTPServer):
    """HTTP Server to work with wsgi applications."""
    def __init__(self, port, wsgi_app):
        application = web.Application(
            (WSGIHandler, wsgi_app)
        )
        httpserver.HTTPServer.__init__(self, application)
        self.listen(port)

    def start(self):
        ioloop.IOLoop.instance().start()        

class WSGIHandler(web.RequestHandler):
    def __init__(self, application, request, wsgi_app):
        self.wsgi_app = wsgi_app
        web.RequestHandler.__init__(self, application, request)

    def delegate(self):
        env = self.make_wsgi_environ(self.request)
        out = self.wsgi_app(env, self._start_response)

        if not (hasattr(out, 'next') or isinstance(out, list)):
            out = [out]

        # don't send any data for redirects
        if self._status_code not in [301, 302, 303, 304, 307]:
            for x in out:
                self.write(x)

    get = post = put = delete = delegate

    def _start_response(self, status, headers, exc_info=None):
        status_code = int(status.split()[0])
        self.set_status(status_code)
        for name, value in headers:
            self.set_header(name, value)

    def make_wsgi_environ(self, request):
        """Makes wsgi environment using HTTPRequest"""
        env = {}
        env['REQUEST_METHOD'] = request.method
        env['SCRIPT_NAME'] = ""
        env['PATH_INFO'] = urllib.unquote(request.path)
        env['QUERY_STRING'] = request.query

        special = ['CONTENT_LENGTH', 'CONTENT_TYPE']

        for k, v in request.headers.items():
            k =  k.upper().replace('-', '_')
            if k not in special:
                k = 'HTTP_' + k
            env[k] = v

        env["wsgi.url_scheme"] = request.protocol
        env['REMOTE_ADDR'] = request.remote_ip
        env['HTTP_HOST'] = request.host
        env['SERVER_PROTOCOL'] = request.version

        if request.body:
            env['wsgi.input'] = StringIO.StringIO(request.body)

        env['wsgi.errors'] = sys.stderr
        env['wsgi.multithread'] = False
        env['wsgi.multiprocess'] = False
        env['wsgi.run_once'] = False

        return env


class SocketLocalMiddleware:
    """WSGI middleware to setup socket-local for request handling socket-thread."""
    def __init__(self, wsgi_app):
        self.wsgi_app = wsgi_app

    def __call__(self, env, start_response):
        ioloop.IOLoop().instance().get_current_thread().local = {}
        return self.wsgi_app(env, start_response)





#coding:utf-8


try:
    from myconf.config import ORG_CSS_JS
except:
    ORG_CSS_JS = False

try:
    from myconf.config import FILE_HOST
except:
    FILE_HOST = ""

if ORG_CSS_JS:

    my = "/css/my.css"

else:

    my = "%s/css/g9Ag~my.css"%FILE_HOST



#! /usr/bin/env python
#coding=utf-8


WHITE_SPACE = "\r\n \t　"

def cnenlen(s):
    return (len(s.decode("utf-8","ignore").encode("gb18030","ignore"))+1)//2

EMAIL_DICT ={
        "msn.com":("MSN","http://login.live.com/"),
        "2008.sina.com":("新浪","http://mail.2008.sina.com.cn/index.html"),
        "sina.com.cn":("新浪","http://mail.sina.com.cn/index.html"),
        "vip.163.com":("163","http://vip.163.com/"),
        "hongkong.com":("中华邮","http://mail.china.com"),
        "sohu.com":("搜狐","http://mail.sohu.com"),
        "live.com":("Live","http://login.live.com/"),
        "eyou.com":("亿邮","http://mail.eyou.com/"),
        "citiz.net":("Citiz","http://citiz.online.sh.cn/citiz_index.htm"),
        "yeah.net":("Yeah","http://mail.yeah.net/"),
        "vip.tom.com":("Tom","http://vip.tom.com/"),
        "vip.qq.com":("QQ","http://mail.qq.com/cgi-bin/loginpage?t=loginpage_vip&f=html"),
        "yahoo.com.hk":("雅虎","http://mail.yahoo.com.hk"),
        "yahoo.cn":("雅虎","http://mail.cn.yahoo.com/"),
        "188.com":("188","http://www.188.com/"),
        "2008.china.com":("中华邮","http://mail.china.com"),
        "vip.sohu.com":("搜狐","http://mail.sohu.com"),
        "163.com":("163","http://mail.163.com"),
        "126.com":("126","http://www.126.com/"),
        "chinaren.com":("chinaren","http://mail.chinaren.com"),
        "tom.com":("Tom","http://mail.tom.com/"),
        "china.com":("中华邮","http://mail.china.com"),
        "139.com":("139","http://mail.139.com/"),
        "hotmail.com":("Hotmail","http://www.hotmail.com"),
        "21cn.com":("21cn","http://mail.21cn.com/"),
        "gmail.com":("Gmail","http://mail.google.com"),
        "my3ia.sina.com":("新浪","http://vip.sina.com.cn/index.html"),
        "yahoo.com.tw":("雅虎","http://mail.yahoo.com.tw"),
        "vip.sina.com":("新浪","http://vip.sina.com.cn/index.html"),
        "mail.china.com":("中华邮","http://mail.china.com"),
        "263.net":("263","http://mail.263.net/"),
        "yahoo.com":("雅虎","https://login.yahoo.com/"),
        "foxmail.com":("Foxmail","http://www.foxmail.com/"),
        "qq.com":("QQ","http://mail.qq.com"),
        "sina.cn":("新浪","http://vip.sina.com.cn/index.html"),
        "yahoo.com.cn":("雅虎","http://mail.cn.yahoo.com/"),
        "sogou.com":("搜狗","http://mail.sogou.com/"),
        "sina.com":("新浪","http://mail.sina.com.cn/index.html"),
        "live.cn":("Live","http://login.live.com/"),
}

import cgi
def email2link(email):
    if email:
        email = cgi.escape(email)
        e_domain = email.split(str('@'))[1]
        link = EMAIL_DICT.get(e_domain, None)
        if link:
            return """<a href="%s" target="_blank">%s</a>"""%(link[1],email)
    return email



#! /usr/bin/env python
#coding=utf-8
import types
import inspect
from http_exc import Http404
from urllib import unquote

class RouteType:
    module , im_class , func , im_func = range(4)


ACCESS_FUNC = "_access"
LOOKUP_FUNC = "_lookup"

class UrlMap(object):
    def __init__(self):
        self.__dict__["_urlmap"] = {}
        self.__dict__["_obj"] = None
        self.__dict__["_type"] = None
        self.__dict__["_parse"] = None
        self.__dict__["_arg_len"] = 0

    def __getattr__(self, name):
        r = self._urlmap.get(name)
        if not r:
            r = UrlMap()
            self._urlmap[name] = r
        return r

    def __lshift__(self, url):
        url = unquote(url)
        url = tuple(i for i in url.split("/") if i)
        url_length = len(url)
        now = self
        pos = 0
        temp = None
        result = None
        while 1:
            if now is None:
                raise Http404
            elif now._obj:
                arg_len = now._arg_len
                if arg_len:
                    pos_next = pos + now._arg_len
                    arg = url[pos:pos_next]
                    pos = pos_next
                else:
                    arg = ()
                is_class = now._type is RouteType.im_class                
                if is_class:
                    ins = now._obj()
                else:
                    is_im_func = now._type is RouteType.im_func 
                
                if now._parse:
                    if is_class or is_im_func:
                        result = now._parse(ins, *arg)
                    else:
                        result = now._parse(*arg)
                    if result is not None:
                        if result is False:
                            return ""
                        return result
                    
            if pos < url_length:
                urlmap = now._urlmap
                name = url[pos]
                
                if name and name[0].isalpha():
                    temp = urlmap.get(name)
                if temp:
                    pos+=1
                    now = temp
                    temp = None
                else:
                    now = urlmap.get("_lookup")
            else:
                break
        
        if now._type is RouteType.module:
            now = now._urlmap.get("index")
            if now and now._parse:
                result = now._parse()
            else:
                raise Http404
        
        elif now._type is RouteType.im_class:
            now = now._urlmap.get("index")
            if now and now._parse:
                result = now._parse(result)
            else:
                raise Http404
        
        if result is False:
            return ""
        return result

    def __call__(self, obj):
        self._obj = obj
        obj_type = type(obj)
         
        if obj_type is types.FunctionType:
            self._type = RouteType.func
            self._parse = obj
            self._arg_len = obj.func_code.co_argcount
        
        elif obj_type is types.MethodType:
            self._type = RouteType.im_func
            self._parse = obj
            self._arg_len = obj.func_code.co_argcount-1
        
        elif obj_type is types.ModuleType:
            self._type = RouteType.module
            if hasattr(obj, ACCESS_FUNC):
                access = getattr(obj, ACCESS_FUNC)
                if type(access) is types.FunctionType:
                    self._parse = access
                    self._arg_len = access.func_code.co_argcount
            if hasattr(obj, LOOKUP_FUNC):
                self._lookup(getattr(obj,LOOKUP_FUNC))
        
        elif inspect.isclass(obj):
            self._type = RouteType.im_class
            if hasattr(obj, ACCESS_FUNC):
                self._obj = obj
                access = getattr(self._obj, ACCESS_FUNC)
                if type(access) is types.MethodType:
                    self._parse = access
                    self._arg_len = access.func_code.co_argcount-1
            if hasattr(obj, LOOKUP_FUNC):
                self._lookup(getattr(obj,LOOKUP_FUNC))
        return obj

def ltrim(string,prefix):
    if string.startswith(prefix):
        return string[len(prefix):]
    return string

class Route(object):
    def __init__(self):
        self.__dict__["_urlmap"] = UrlMap()
        self.__dict__["_buf"] = []

    def __lshift__(self, url):
        return self._urlmap<<url

    def __getattr__(self, name):
        return getattr(self._urlmap, name)

    def __call__(self, obj,namespace=[]):
        self._buf.append((obj,namespace))
        return obj

    def _install(self, app,prefix=''):
        self._urlmap(app)
        for obj,namespace in self._buf:
            obj_type = type(obj)
            if obj_type in (types.FunctionType, types.MethodType):
                obj_type = type(obj)

                def mod2list(o):
                    r = o.__module__
                    r = ltrim(r,prefix)
                    r = r[len(ltrim(self._urlmap._obj.__name__,prefix)):]
                    r = r.lstrip(".").split(".")+namespace
                    return r
                
                if obj_type is types.FunctionType:
                    obj_prefix = mod2list(obj)
                    obj_name = obj.__name__
                elif obj_type is types.MethodType:
                    obj_prefix = mod2list(obj.im_func)
                    obj_name = obj.im_func.__name__
                obj_prefix.append(obj_name)
                
                now = self._urlmap
                for i in obj_prefix:
                    #print obj_prefix
                    if i:
                        next_urlmap = getattr(now, i)
                        if not next_urlmap._obj:
                            try:
                                next_urlmap(getattr(now._obj, i))
                            except:
                                print now._obj,i
                                raise
                        now = next_urlmap

            elif inspect.isclass(obj):
                for k in obj.__dict__:
                    if not k.startswith("_"):
                        self(getattr(obj, k),namespace+[obj.__name__])
        self._buf = []


#coding:utf-8

from os.path import abspath,dirname,join,normpath
import sys

#初始化python的查找路径
PREFIX = normpath(dirname(dirname(dirname(abspath(__file__)))))
if PREFIX not in sys.path:
    sys.path=[PREFIX] + sys.path


from myconf.config import MEMCACHED_ADDR,DISABLE_LOCAL_CACHED
from sqlbean.db.mc_connection import init_mc
init_mc(memcached_addr=MEMCACHED_ADDR,disable_local_cached=DISABLE_LOCAL_CACHED)


from myconf.config import DATABASE_CONFIG,THREAD_SAFE
from sqlbean.db import connection
connection.THREAD_SAFE = THREAD_SAFE

from sqlbean.db import sqlstore
SQLSTORE = sqlstore.SqlStore(db_config=DATABASE_CONFIG, **{})
def get_db_by_table(table_name):
    return SQLSTORE.get_db_by_table(table_name)
connection.get_db_by_table = get_db_by_table

from sqlbean.shortcut import Query,mc,MMcCache,McCache,ForeignKey,OneToMany,Model,McModel





#!/usr/bin/env python
#coding:utf-8
from __future__ import with_statement
from os.path import dirname, join, abspath, basename
import subprocess
from os import makedirs
from os.path import exists
from hashlib import md5
from os import walk
from glob import glob
from struct import unpack
from zlib import crc32
from shutil  import copyfile
from struct import pack
from base64 import urlsafe_b64encode


prefix = dirname(abspath(__file__))
merge_prefix = prefix

def walk_filter(basedir, dirfilter=None, filefilter=None):
    for dirpath, dirnames, filenames in walk(basedir):
        if dirfilter and not dirfilter(dirpath):
            continue
        prefix = dirpath[len(basedir)+1:]
        for filename in filenames:
            if filefilter and not filefilter(filename):
                continue
            yield join(prefix, filename)

class ChangeFilter():
    def __init__(self,buffer_file):
        self.filename2num = dict()
        self.filename2hash = dict()
        self.buffer_file = buffer_file
        self.salt = ""

    def __call__(self, basedir, filelist):
        buffer_file = self.buffer_file
        if exists(buffer_file):
            init_py = open(buffer_file)
            hash_lines = [
                i.strip() for i in
                list(init_py)
            ]
            hash_lines = [
                i for i in hash_lines if i
            ]
            init_py.close()
        else:
            hash_lines = []
        
        if hash_lines:
            self.salt = hash_lines[0]
            hash_lines = hash_lines[1:]
        else:
            
            from os import urandom
            self.salt = urlsafe_b64encode(urandom(3))[:3]
            
            
        for i in hash_lines:
            hash, num, filename = i.split()
            self.filename2hash[filename] = hash
            self.filename2num[filename] = int(num)
        
        for i in filelist:
            with open(join(basedir, i)) as inf:
                hash = md5(inf.read()).hexdigest()
                result = self.filename2hash.get(i)
                if result == hash:
                    continue
                if result is None:
                    self.filename2num[i] = 1
                else:
                    self.filename2num[i] += 1
                self.filename2hash[i] = hash
                yield i
    
    def flush(self):
        with open(self.buffer_file, "w") as inf:
            inf.write("%s\n"%self.salt)
            for filename, hash in self.filename2hash.iteritems():
                inf.write(
                    "%s\t%s\t%s\n"%(hash, self.filename2num[filename], filename)
                )

def merge(subdir,template):
    file_hash_version = ChangeFilter(
        join(prefix, ".%s_hash"%subdir)
    )
    subdir=join(prefix,subdir)
    for i in file_hash_version(
        subdir,
        walk_filter(subdir, lambda x:x.find(".svn")==-1, lambda x:(x.endswith(".css") or x.endswith(".js")) and not x.startswith("."))
    ):
        infile = join(subdir, i)
        bi = basename(i)
        outfile = join(subdir, i[:-len(bi)]+"."+bi)
        filetype = "css"
        if infile.endswith(".js"):
            filetype = "js"
        cmd = [
                "java","-jar",join(prefix,"yuicompressor.jar"),"--charset=utf-8","--type",filetype,infile,"-o",outfile
        ]
        try:
            returncode = subprocess.call(cmd)
            if returncode:
                raise
        except:
            file_hash_version.filename2num[i]-=1
            file_hash_version.filename2hash[i]="0"
            print "yuicompressor error : %s"%infile
            copyfile(infile,outfile)
    file_hash_version.flush()
    from mako.template import Template
    init_template = Template(template)
    init_py_file = join(subdir, "__init__.py")
    with open(init_py_file, "w") as init_py:
        filenum = file_hash_version.filename2num.iteritems()
        init_py.write(
            "#coding:utf-8\n\n"+init_template.render(
                filename2num = dict(
                    (
                        k,
                         "%s%s"%(
                            file_hash_version.salt,
                            urlsafe_b64encode(
                                pack("Q",v).rstrip("\x00")
                            ).rstrip("=")
                        )
                    )
                    for k,v in 
                    file_hash_version.filename2num.iteritems()
                )
            )
        )

JS_INIT_TEMPLATE = """\
<%!
from os.path import join,basename,dirname
%>
try:
    from myconf.config import ORG_CSS_JS
except:
    ORG_CSS_JS = False

try:
    from myconf.config import JS_FILE_HOST
except:
    JS_FILE_HOST = ""

if ORG_CSS_JS:
%for path,num in filename2num.iteritems():
<%
path = path.rsplit(".js",1)[0]
name = path.replace("/","_").replace(".","_")
%>
    ${name} = "/js/${path}.js"
%endfor

else:
%for path,num in filename2num.iteritems():
<%
path = path.rsplit(".js",1)[0]
name = path.replace("/","_").replace(".","_")
pathdir = join("js",dirname(path))
pathfile = basename(path)
filename = join(pathdir,"%s~%s.js"%(num,pathfile)).replace("\\\\","/")
%>
    ${name} = "%s/${filename}"%JS_FILE_HOST
%endfor
"""


CSS_INIT_TEMPLATE = """\
<%!
from os.path import join,basename,dirname
%>
try:
    from myconf.config import ORG_CSS_JS
except:
    ORG_CSS_JS = False

try:
    from myconf.config import FILE_HOST
except:
    FILE_HOST = ""

if ORG_CSS_JS:
%for path,num in filename2num.iteritems():
<%
path = path.rsplit(".css",1)[0]
name = path.replace("/","_").replace(".","_")
%>
    ${name} = "/css/${path}.css"
%endfor

else:
%for path,num in filename2num.iteritems():
<%
path = path.rsplit(".css",1)[0]
name = path.replace("/","_").replace(".","_")
pathdir = join("css",dirname(path))
pathfile = basename(path)
filename = join(pathdir,"%s~%s.css"%(num,pathfile)).replace("\\\\","/")
%>
    ${name} = "%s/${filename}"%FILE_HOST
%endfor
"""
merge("css",CSS_INIT_TEMPLATE)
print "CSS Merge Game Over"

merge("js",JS_INIT_TEMPLATE)
print "JS Merge Game Over"



#!/usr/bin/env python2.4
"""yaro - Yet Another Request Object (for WSGI)

A simple but non-restrictive abstraction of WSGI for end users.

(See the docstrings of the various functions and classes.)

Copyright (C) 2006-2007 Luke Arno - http://lukearno.com/

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to:

The Free Software Foundation, Inc., 
51 Franklin Street, Fifth Floor, 
Boston, MA  02110-1301, USA.

Luke Arno can be found at http://lukearno.com/

"""

import urllib
import cgi
import mimetypes
import Cookie
from cStringIO import StringIO
from dict_like import Jsdict
from wsgiref import headers, util


class URI(object):
    """URI info from a WSGI environ."""

    def __init__(self, environ):
        """Just pass in environ to populate."""
        self.scheme = environ['wsgi.url_scheme']
        if environ.get('HTTP_HOST'):
            self.host = environ['HTTP_HOST']
            if ':' in self.host:
                self.host, self.port = self.host.split(':')
            else:
                if self.scheme == 'http':
                    self.port = '80'
                else:
                    self.port = '443'
        else:
            self.host += environ['SERVER_NAME']
            self.port += environ['SERVER_PORT']
        self.script = environ.get('SCRIPT_NAME', '')
        self.path = environ.get('PATH_INFO', '')
        self.query = environ.get('QUERY_STRING', '')

    def __call__(self, path=None, with_qs=False):
        """Complete URI string with optional query string and path.
        
        path=None and will default to self.path (from PATH_INFO)
        with_qs=False and requests inclusion or not of query string.

        This is handy for doing redirects within your application
        or generating form
        """
        if path is None:
            path = self.path
        if path == '':
            uri = self.application_uri()
        elif path[0:1] != '/':
            uri = self.application_uri()
            uri = '/'.join((uri + self.path).split('/')[:-1])
            while path[:3] == '../':
                uri = '/'.join(uri.split('/')[:-1])
                path = path[3:]
            path = '/' + path
        else:
            uri = self.server_uri()
        uri += urllib.quote(path)
        if with_qs and self.query:
            uri += '?' + self.query
        return uri

    def server_uri(self):
        """URI of server with no script_name or path_info or query_string."""
        uri = self.scheme + '://' + self.host
        if self.scheme + self.port not in ('http80', 'https443'):
            uri += ':' + self.port
        return uri

    def application_uri(self):
        """URI up to and including script_name."""
        return self.server_uri() + urllib.quote(self.script)


class Request(object):
    """Yet another request object (for WSGI), as advertised."""

    # all the expected members other than res and start_response_called
    # these are the attrs which we do not want to cache
    _volitile_attrs = ['__class__',
                       '__delattr__',
                       '__dict__',
                       '__doc__',
                       '__getattr__',
                       '__getattribute__',
                       '__hash__',
                       '__init__',
                       '__module__',
                       '__new__',
                       '__reduce__',
                       '__reduce_ex__',
                       '__repr__',
                       '__setattr__',
                       '__str__',
                       '__weakref__',
                       '_load_body',
                       '_parse_form',
                       '_parse_query',
                       '_start_response',
                       'content_length',
                       'content_type',
                       'environ',
                       'exc_info',
                       'extra_props',
                       'forward',
                       'load_prop',
                       'method',
                       'query',
                       'redirect',
                       'start_response',
                       'uri',
                       'wsgi_forward']

    def save_to_environ(self):
        """Save values of attrs in environ.
        
        Fields in ._volitile_attrs and .extra_props are _not_
        persisted to environ.
        """
        dct = self.environ.get('yaro.cached_attrs', {})
        for attr in dir(self):
            if attr not in (self._volitile_attrs 
                             + [p[0] for p in self.extra_props or []]):
                dct[attr] = getattr(self, attr)
        self.environ['yaro.cached_attrs'] = dct

    def load_from_environ(self):
        """Load values of attrs cached in environ.
        
        Fields in ._volitile_attrs and .extra_props are _not_
        persisted to environ.
        """
        self.__dict__.update(self.environ.get('yaro.cached_attrs', {}))

    def __init__(self, environ, start_response, extra_props=None):
        """Set up the various attributes."""
        self.environ = environ
        self._start_response = start_response
        self.start_response_called = False
        self.method = environ['REQUEST_METHOD']
        self.content_type = environ.get('CONTENT_TYPE', '')
        self.content_length = environ.get('CONTENT_LENGTH', '')
        self.uri = URI(environ)
        self.res = Response(self.uri())
        self._parse_query()
        self.extra_props = extra_props
        if extra_props is not None:
            for prop_spec in extra_props:
                self.load_prop(*prop_spec)
        self.exc_info = None
        self.load_from_environ()
        self.save_to_environ()

        cookie = Cookie.SimpleCookie()
        cookie.load(self.environ.get('HTTP_COOKIE', ''))
        cookie_dict = {}
        for k,v in cookie.iteritems():
            cookie_dict[k]=v.value 
        self.cookie = cookie_dict

    def start_response(self, *a, **kw):
        """Wrap the real start_response and set flag when called.
        
        This provides a means to prevent the real start_response from 
        getting called more than once.

        The flag is self.start_response_called = True|False.
        """
        self.start_response_called = True
        return self._start_response(*a, **kw)

    def load_prop(self, attr_name, key, default=None):
        """Add an arbitrary property."""
        if isinstance(key, str):
            value = self.environ.get(key, default)
        else:
            value = key(self.environ)
        setattr(self, attr_name, value)

    def forward(self, app):
        """Forward the request to another Yaro compatible handler."""
        return app(self)

    def wsgi_forward(self, wsgiapp):
        """Forward the request to a WSGI compatible handler."""
        self.save_to_environ()
        return wsgiapp(self.environ, self.start_response)

    def __getattr__(self, attr):
        """Support lazy loading of form or body."""
        if attr == 'form' and not 'form' in self.__dict__:
            self._parse_form()
        elif attr == 'body' and not 'body' in self.__dict__:
            self._load_body()
        elif attr == 'is_post' and not 'is_post' in self.__dict__:
            self.is_post = (self.method == "POST")
        return self.__dict__[attr]

    def _parse_query(self):
        """Populate a dictionary in self.query from the querystring.
        
        The dictionary created will have strings or lists of strings
        as its values.
        
        If you have a parameter like ?foo[]=bar then 
        self.query['foo'] == ['bar'] ('bar' will be in a list even if 
        it has only one member). The key will be 'foo'.

        In other words, this is a way to indicate that you want a 
        list and avoid boilerplate.

        When a name does not end in brackets, you will end up with a 
        list only if there is more than one parameter by that name.

        If 'foo[]' and 'foo' are both used as input names, they will 
        step on each other. Use one or the other.
        """
        qu = cgi.parse_qs(self.uri.query)
        query = Jsdict()
        for key, value in qu.iteritems():
            if key.endswith('[]'):
                if not isinstance(value, list):
                    value = [value]
                query[key[:-2]] = value
            else:
                if isinstance(value, list) and len(value) == 1:
                    value = value[0]
                query[key] = value
        self.query = query

    def _parse_form(self):
        """Populate a dictionary in self.form from webform data.
        
        The dictionary created will have strings, cgi.FieldStorage 
        objects (used for file uploads) or lists of strings and/or 
        cgi.FieldStorage objects as its values.
        
        If your form data has an field named "foo[]", its value 
        will be found in self.form in a list even if it has only 
        one member. The key will be 'foo'.

        In other words, this is a way to indicate that you want a
        list and avoid boilerplate.

        When a name does not end in brackets, you will end up with 
        a list only if there is more than one field by that name.

        If 'foo[]' and 'foo' are both used as input names, they 
        will step on each other. Use one or the other.
        """
        form = Jsdict()
        clen = self.environ.get('CONTENT_LENGTH', 0)
        if clen:
            sio = StringIO(self.environ['wsgi.input'].read(int(clen)))
            fs = cgi.FieldStorage(fp=sio,
                                  environ=self.environ,
                                  keep_blank_values=True) 
            sio.seek(0)
            self.environ['wsgi.input'] = sio
            for key in fs:
                value = fs[key]
                if key.endswith("[]"):
                    if not isinstance(value, list):
                        value = [value]
                    key = key[:-2]
                if isinstance(value, list):
                    newvalue = []
                    for v in value:
                        if v.filename:
                            newvalue.append(v)
                        else:
                            newvalue.append(v.value)
                    form[key] = newvalue
                else:
                    if value.filename is None:
                        value = value.value
                    form[key] = value
        self.form = form

    def _load_body(self):
        """Set self.body with the raw body of the request."""
        clen = self.environ.get('CONTENT_LENGTH', 0)
        if clen:
            self.body = self.environ['wsgi.input'].read(int(clen))
        else:
            self.body = ""


    def redirect(self, location, permanent=False):
        """Set the location header and status."""
        self.res.headers['Location'] = location
        if permanent is True:
            self.res.status = "301 Moved Permanently" 
        else:
            self.res.status = "302 Found"
        return False

class Response(object):
    """Hold on to info about the response to a request."""

    def __init__(self, uri):
        """Set up various useful defaults."""
        self._headers = []
        self.headers = headers.Headers(self._headers)
        guess = mimetypes.guess_type(uri)[0]
        self.headers['Content-Type'] = guess or 'text/html'
        self.status = '200 OK'
        self.body = ""


class Yaro(object):
    """WSGI wrapper for something that takes and returns Request."""

    def __init__(self, app, extra_props=None):
        """Take the thing to wrap."""
        self.app = app
        self.extra_props = extra_props

    def __call__(self, environ, start_response):
        """Create Request, call thing, unwrap results and respond."""
        req = Request(environ, start_response, self.extra_props)
        body = self.app(req)
        req.save_to_environ()
        if body is None:
            body = req.res.body
        if not req.start_response_called:
            req.start_response(req.res.status, req.res._headers, req.exc_info)
            req.start_response_called = True
        if isinstance(body, str):
            return [body]
        elif isiterable(body):
            return body
        else:
            return util.FileWrapper(body)


class OYaro(object):
    """WSGI wrapper for something that takes and returns Request.
    
    For use with bound methods (things that need a reference to the
    instance of which they are a member when called).
    """

    def __init__(self, app, extra_props=None):
        """Take the thing to wrap."""
        self.app = app
        self.extra_props = extra_props

    def __call__(self, instance, environ, start_response):
        """Create Request, call thing, unwrap results and respond."""
        req = Request(environ, start_response, self.extra_props)
        body = self.app(instance, req)
        req.save_to_environ()
        if body is None:
            body = req.res.body
        if not req.start_response_called:
            req.start_response(req.res.status, req.res._headers, req.exc_info)
            req.start_response_called = True
        if isinstance(body, str):
            return [body]
        elif isiterable(body):
            return body
        else:
            return util.FileWrapper(body)


def oYaro(extra_props=None):
    """Decorate a bound method so that it calls an OYaro instance."""
    def oy_decorator(fn):
        oy = OYaro(fn, extra_props)
        def newfn(self, environ, start_response):
            return oy(self, environ, start_response)
        newfn.__name__ = fn.__name__
        newfn.__dict__ = fn.__dict__
        newfn.__doc__ = fn.__doc__
        return newfn
    return oy_decorator


def isiterable(it):
    """Return True if 'it' is iterable else return False."""
    try:
        iter(it)
    except:
        return False
    else:
        return True


#  'start_response_called', 


if __name__ == '__main__':

    import code
    
    from wsgiref.simple_server import make_server

    def foo(req):
        #req.res.body = 'Hello, World!'
        sc = Cookie.SimpleCookie(); sc['mycookie'] = 'cookieval'
        req.res.headers.add_header(*tuple(str(sc).split(': ', 1)))
        code.interact(local=locals(), banner="%s: %s" % (req.method, req.uri()))
        if req.uri.path.endswith('.rdr'):
            req.redirect('../proudhon.txt')
        elif req.uri.path.endswith('.fwd'):
            return req.forward(bar)
        elif req.uri.path.endswith('.wsgi'):
            return req.wsgi_forward(baz)
        else:
            req.res.body = req.uri()

    def bar(req):
        return "Hello Bar"

    def baz(environ, start_response):
        start_response("201 Created", [])
        return ['Baz here.']

    def boing(req):
        req.stuff = 'This is stuff you should see.'
        return req.wsgi_forward(Yaro(zoink))

    def zoink(req):
        return req.stuff

    class OYaroTestApp(object):
        @oYaro()
        def hi(self, req):
            return "Hi, my name is %s" % self.__class__.__name__

    try:
        make_server('localhost', 9999, OYaroTestApp().hi).serve_forever()
    except KeyboardInterrupt, ki:
        print 'I said "Good day!"'



#!/usr/bin/env python
#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""A level-triggered I/O loop for non-blocking sockets."""

import bisect
import errno
import fcntl
import logging
import os
import select
import time


class IOLoop(object):
    """A level-triggered I/O loop.

    We use epoll if it is available, or else we fall back on select(). If
    you are implementing a system that needs to handle 1000s of simultaneous
    connections, you should use Linux and either compile our epoll module or
    use Python 2.6+ to get epoll support.

    Example usage for a simple TCP server:

        import errno
        import functools
        import ioloop
        import socket

        def connection_ready(sock, fd, events):
            while True:
                try:
                    connection, address = sock.accept()
                except socket.error, e:
                    if e[0] not in (errno.EWOULDBLOCK, errno.EAGAIN):
                        raise
                    return
                connection.setblocking(0)
                handle_connection(connection, address)

        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.setblocking(0)
        sock.bind(("", port))
        sock.listen(128)

        io_loop = ioloop.IOLoop.instance()
        callback = functools.partial(connection_ready, sock)
        io_loop.add_handler(sock.fileno(), callback, io_loop.READ)
        io_loop.start()

    """
    # Constants from the epoll module
    _EPOLLIN = 0x001
    _EPOLLPRI = 0x002
    _EPOLLOUT = 0x004
    _EPOLLERR = 0x008
    _EPOLLHUP = 0x010
    _EPOLLRDHUP = 0x2000
    _EPOLLONESHOT = (1 << 30)
    _EPOLLET = (1 << 31)

    # Our events map exactly to the epoll events
    NONE = 0
    READ = _EPOLLIN
    WRITE = _EPOLLOUT
    ERROR = _EPOLLERR | _EPOLLHUP | _EPOLLRDHUP

    def __init__(self, impl=None):
        self._impl = impl or _poll()
        self._handlers = {}
        self._events = {}
        self._callbacks = set()
        self._timeouts = []
        self._running = False

        # Create a pipe that we send bogus data to when we want to wake
        # the I/O loop when it is idle
        r, w = os.pipe()
        self._set_nonblocking(r)
        self._set_nonblocking(w)
        self._waker_reader = os.fdopen(r, "r", 0)
        self._waker_writer = os.fdopen(w, "w", 0)
        self.add_handler(r, self._read_waker, self.WRITE)

    @classmethod
    def instance(cls):
        """Returns a global IOLoop instance.

        Most single-threaded applications have a single, global IOLoop.
        Use this method instead of passing around IOLoop instances 
        throughout your code.

        A common pattern for classes that depend on IOLoops is to use
        a default argument to enable programs with multiple IOLoops
        but not require the argument for simpler applications:

            class MyClass(object):
                def __init__(self, io_loop=None):
                    self.io_loop = io_loop or IOLoop.instance()
        """
        if not hasattr(cls, "_instance"):
            cls._instance = cls()
        return cls._instance

    def add_handler(self, fd, handler, events):
        """Registers the given handler to receive the given events for fd."""
        self._handlers[fd] = handler
        self._impl.register(fd, events | self.ERROR)

    def update_handler(self, fd, events):
        """Changes the events we listen for fd."""
        self._impl.modify(fd, events | self.ERROR)

    def remove_handler(self, fd):
        """Stop listening for events on fd."""
        self._handlers.pop(fd, None)
        self._events.pop(fd, None)
        try:
            self._impl.unregister(fd)
        except OSError:
            logging.debug("Error deleting fd from IOLoop", exc_info=True)

    def start(self):
        """Starts the I/O loop.

        The loop will run until one of the I/O handlers calls stop(), which
        will make the loop stop after the current event iteration completes.
        """
        self._running = True
        while True:
            # Never use an infinite timeout here - it can stall epoll
            poll_timeout = 0.2

            # Prevent IO event starvation by delaying new callbacks
            # to the next iteration of the event loop.
            callbacks = list(self._callbacks)
            for callback in callbacks:
                # A callback can add or remove other callbacks
                if callback in self._callbacks:
                    self._callbacks.remove(callback)
                    self._run_callback(callback)

            if self._callbacks:
                poll_timeout = 0.0

            if self._timeouts:
                now = time.time()
                while self._timeouts and self._timeouts[0].deadline <= now:
                    timeout = self._timeouts.pop(0)
                    self._run_callback(timeout.callback)
                if self._timeouts:
                    milliseconds = self._timeouts[0].deadline - now
                    poll_timeout = min(milliseconds, poll_timeout)

            if not self._running:
                break

            try:
                event_pairs = self._impl.poll(poll_timeout)
            except Exception, e:
                if e.args == (4, "Interrupted system call"):
                    logging.warning("Interrupted system call", exc_info=1)
                    continue
                else:
                    raise

            # Pop one fd at a time from the set of pending fds and run
            # its handler. Since that handler may perform actions on
            # other file descriptors, there may be reentrant calls to
            # this IOLoop that update self._events
            self._events.update(event_pairs)
            while self._events:
                fd, events = self._events.popitem()
                try:
                    self._handlers[fd](fd, events)
                except KeyboardInterrupt:
                    raise
                except OSError, e:
                    if e[0] == errno.EPIPE:
                        # Happens when the client closes the connection
                        pass
                    else:
                        logging.error("Exception in I/O handler for fd %d",
                                      fd, exc_info=True)
                except:
                    logging.error("Exception in I/O handler for fd %d",
                                  fd, exc_info=True)

    def stop(self):
        """Stop the loop after the current event loop iteration is complete."""
        self._running = False
        self._wake()

    def running(self):
        """Returns true if this IOLoop is currently running."""
        return self._running

    def add_timeout(self, deadline, callback):
        """Calls the given callback at the time deadline from the I/O loop."""
        timeout = _Timeout(deadline, callback)
        bisect.insort(self._timeouts, timeout)
        return timeout

    def remove_timeout(self, timeout):
        self._timeouts.remove(timeout)

    def add_callback(self, callback):
        """Calls the given callback on the next I/O loop iteration."""
        self._callbacks.add(callback)
        self._wake()

    def remove_callback(self, callback):
        """Removes the given callback from the next I/O loop iteration."""
        self._callbacks.pop(callback)

    def _wake(self):
        try:
            self._waker_writer.write("x")
        except IOError:
            pass

    def _run_callback(self, callback):
        try:
            callback()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            logging.error("Exception in callback %r", callback, exc_info=True)

    def _read_waker(self, fd, events):
        try:
            while True:
                self._waker_reader.read()
        except IOError:
            pass

    def _set_nonblocking(self, fd):
        flags = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)


class _Timeout(object):
    """An IOLoop timeout, a UNIX timestamp and a callback"""
    def __init__(self, deadline, callback):
        self.deadline = deadline
        self.callback = callback

    def __cmp__(self, other):
        return cmp((self.deadline, id(self.callback)),
                   (other.deadline, id(other.callback)))


class PeriodicCallback(object):
    """Schedules the given callback to be called periodically.

    The callback is called every callback_time milliseconds.
    """
    def __init__(self, callback, callback_time, io_loop=None):
        self.callback = callback
        self.callback_time = callback_time
        self.io_loop = io_loop or ioloop.IOLoop.instance()
        self._running = True

    def start(self):
        timeout = time.time() + self.callback_time / 1000.0
        self.io_loop.add_timeout(timeout, self._run)

    def stop(self):
        self._running = False

    def _run(self):
        if not self._running: return
        try:
            self.callback()
        except:
            logging.error("Error in periodic callback", exc_info=True)
        self.start()


class _EPoll(object):
    """An epoll-based event loop using our C module for Python 2.5 systems"""
    _EPOLL_CTL_ADD = 1
    _EPOLL_CTL_DEL = 2
    _EPOLL_CTL_MOD = 3

    def __init__(self):
        self._epoll_fd = epoll.epoll_create()

    def register(self, fd, events):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_ADD, fd, events)

    def modify(self, fd, events):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_MOD, fd, events)

    def unregister(self, fd):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_DEL, fd, 0)

    def poll(self, timeout):
        return epoll.epoll_wait(self._epoll_fd, int(timeout * 1000))


class _Select(object):
    """A simple, select()-based IOLoop implementation for non-Linux systems"""
    def __init__(self):
        self.read_fds = set()
        self.write_fds = set()
        self.error_fds = set()
        self.fd_sets = (self.read_fds, self.write_fds, self.error_fds)

    def register(self, fd, events):
        if events & IOLoop.READ: self.read_fds.add(fd)
        if events & IOLoop.WRITE: self.write_fds.add(fd)
        if events & IOLoop.ERROR: self.error_fds.add(fd)

    def modify(self, fd, events):
        self.unregister(fd)
        self.register(fd, events)

    def unregister(self, fd):
        self.read_fds.discard(fd)
        self.write_fds.discard(fd)
        self.error_fds.discard(fd)

    def poll(self, timeout):
        readable, writeable, errors = select.select(
            self.read_fds, self.write_fds, self.error_fds, timeout)
        events = {}
        for fd in readable:
            events[fd] = events.get(fd, 0) | IOLoop.READ
        for fd in writeable:
            events[fd] = events.get(fd, 0) | IOLoop.WRITE
        for fd in errors:
            events[fd] = events.get(fd, 0) | IOLoop.ERROR
        return events.items()


# Choose a poll implementation. Use epoll if it is available, fall back to
# select() for non-Linux platforms
if hasattr(select, "epoll"):
    # Python 2.6+ on Linux
    _poll = select.epoll
else:
    try:
        # Linux systems with our C module installed
        import epoll
        _poll = _EPoll
    except:
        # All other systems
        import sys
        if "linux" in sys.platform:
            logging.warning("epoll module not found; using select()")
        _poll = _Select



import sys
from shutil import copytree
from os.path import join,dirname

PREFIX = dirname(__file__)


def ignore_new(dirname,filelist):
    ignore = []
    for i in filelist:
        if i==".svn" or i.endswith(".pyc"):
            ignore.append(i)
    return ignore

def runcmd(argv):
    len_argv = len(argv)
    if not argv:
        return
    cmd = argv[0]
    if cmd == "new":
        if len_argv>=2:
            name = argv[1]
            t = join(PREFIX,"project_template")
            copytree(t,name,ignore=ignore_new)

runcmd(sys.argv[1:])


from init_app import application
from myconf import config

try:
    from mypy.tornado.wsgiserver import WSGIServer as WSGIServer
except ImportError:
    print "import tornado.wsgiserver FAILED , USE CherryPyWSGIServer INSTEAD"
    from mypy.wsgiserver import CherryPyWSGIServer as WSGIServer

if config.SERVER_STATIC_FILE:
    from mypy import static
    from os.path import join
    import re
    
    STATIC_VERSION = re.compile("/\d+?~")
    STATIC_FILE = static.Cling(join(config.PREFIX,"myfile"))
    STATIC_PATH = ('/css/','/js/','/pic/','/fs/','/favicon.ico','/bazs.cert','/robots.txt')
    def url_selector(func):
        def _url_selector(environ, start_response):
            path = environ['PATH_INFO']
            for i in STATIC_PATH:
                if path.startswith(i):
                    if i in ('/css/','/js/'):
                        environ['PATH_INFO']=STATIC_VERSION.sub("/.",path)
                    return STATIC_FILE(environ, start_response)

            return func(environ, start_response)
        return _url_selector

    application = url_selector(application)

def run(port=9888):
    print "server on port %s"%port
    server = WSGIServer(port,application)
    server.start()

if __name__=="__main__":
    import sys
    if len(sys.argv)>1 and sys.argv[1].isdigit():
        port = int(sys.argv[1])
    else:
        port = 9888
    run(port=port)



#coding:utf-8
from os import urandom
from base64 import urlsafe_b64encode,urlsafe_b64decode

def get_b64ck():
    return urlsafe_b64encode(urandom(6))

def get_ck():
    return urandom(6)

def decode_b64ck(key):
    try:
        return  urlsafe_b64decode(key)
    except TypeError:
        pass



#!/usr/bin/env python
#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""The Tornado web framework.

The Tornado web framework looks a bit like web.py (http://webpy.org/) or
Google's webapp (http://code.google.com/appengine/docs/python/tools/webapp/),
but with additional tools and optimizations to take advantage of the
Tornado non-blocking web server and tools.

Here is the canonical "Hello, world" example app:

    import tornado.httpserver
    import tornado.ioloop
    import tornado.web

    class MainHandler(tornado.web.RequestHandler):
        def get(self):
            self.write("Hello, world")

    if __name__ == "__main__":
        application = tornado.web.Application([
            (r"/", MainHandler),
        ])
        http_server = tornado.httpserver.HTTPServer(application)
        http_server.listen(8888)
        tornado.ioloop.IOLoop.instance().start()

See the Tornado walkthrough on GitHub for more details and a good
getting started guide.
"""

import calendar
import datetime
import email.utils
import escape
import functools
import hashlib
import httplib
import logging
import os.path
import re


class RequestHandler(object):
    """Subclass this class and define get() or post() to make a handler."""

    def __init__(self, application, request, transforms=None):
        self.application = application
        self.request = request
        self._headers_written = False
        self._finished = False
        self._auto_finish = True
        self._transforms = transforms or []
        self.clear()
    """
    def head(self, *args, **kwargs):
        raise HTTPError(405)

    def get(self, *args, **kwargs):
        raise HTTPError(405)

    def post(self, *args, **kwargs):
        raise HTTPError(405)

    def delete(self, *args, **kwargs):
        raise HTTPError(405)

    def put(self, *args, **kwargs):
        raise HTTPError(405)
    """

    def prepare(self):
        """Called before the actual handler method.

        Useful to override in a handler if you want a common bottleneck for
        all of your requests.
        """
        pass

    def clear(self):
        """Resets all headers and content for this response."""
        self._headers = {
            #"Server": "TornadoServer/0.1",
            "Content-Type": "text/html; charset=UTF-8",
        }
        if not self.request.supports_http_1_1():
            if self.request.headers.get("Connection") == "Keep-Alive":
                self.set_header("Connection", "Keep-Alive")
        self._write_buffer = []
        self._status_code = 200

    def set_status(self, status_code):
        """Sets the status code for our response."""
        assert status_code in httplib.responses
        self._status_code = status_code

    def set_header(self, name, value):
        """Sets the given response header name and value.

        If a datetime is given, we automatically format it according to the
        HTTP specification. If the value is not a string, we convert it to
        a string. All header values are then encoded as UTF-8.
        """
        if isinstance(value, datetime.datetime):
            t = calendar.timegm(value.utctimetuple())
            value = email.utils.formatdate(t, localtime=False, usegmt=True)
        elif isinstance(value, int) or isinstance(value, long):
            value = str(value)
        else:
            value = str(value)
            # If \n is allowed into the header, it is possible to inject
            # additional headers or split the request. Also cap length to
            # prevent obviously erroneous values.
            safe_value = re.sub(r"[\x00-\x1f]", " ", value)[:4000]
            if safe_value != value:
                raise ValueError("Unsafe header value %r", value)
        self._headers[name] = value


    def write(self, chunk):
        """Writes the given chunk to the output buffer.

        To write the output to the network, use the flush() method below.

        If the given chunk is a dictionary, we write it as JSON and set
        the Content-Type of the response to be text/javascript.
        """
        assert not self._finished
        if isinstance(chunk, dict):
            chunk = escape.json_encode(chunk)
            self.set_header("Content-Type", "text/javascript; charset=UTF-8")
        self._write_buffer.append(chunk)


    def flush(self, include_footers=False):
        """Flushes the current output buffer to the nextwork."""
        if not self._headers_written:
            self._headers_written = True
            headers = self._generate_headers()
        else:
            headers = ""

        # Ignore the chunk and only write the headers for HEAD requests
        if self.request.method == "HEAD":
            if headers: self.request.write(headers)
            return

        if self._write_buffer:
            chunk = "".join(self._write_buffer)
            self._write_buffer = []
        else:
            chunk = ""

        if headers or chunk:
            self.request.write(headers + chunk)

    def finish(self, chunk=None):
        """Finishes this response, ending the HTTP request."""
        assert not self._finished
        if chunk: self.write(chunk)

        # Automatically support ETags and add the Content-Length header if
        # we have not flushed any content yet.
        if not self._headers_written:
            if self._status_code == 200 and self.request.method == "GET":
                hasher = hashlib.sha1()
                for part in self._write_buffer:
                    hasher.update(part)
                etag = '"%s"' % hasher.hexdigest()
                inm = self.request.headers.get("If-None-Match")
                if inm and inm.find(etag) != -1:
                    self._write_buffer = []
                    self.set_status(304)
                else:
                    self.set_header("Etag", etag)
            if "Content-Length" not in self._headers:
                content_length = sum(len(part) for part in self._write_buffer)
                self.set_header("Content-Length", content_length)

        self.flush(include_footers=True)
        self.request.finish()
        self._log()
        self._finished = True

    def send_error(self, status_code=500):
        """Sends the given HTTP error code to the browser.

        We also send the error HTML for the given error code as returned by
        get_error_html. Override that method if you want custom error pages
        for your application.
        """
        if self._headers_written:
            logging.error("Cannot send error response after headers written")
            if not self._finished:
                self.finish()
            return
        self.clear()
        self.set_status(status_code)
        message = self.get_error_html(status_code)
        self.finish(message)

    def get_error_html(self, status_code):
        """Override to implement custom error pages."""
        return "<html><title>%(code)d: %(message)s</title>" \
               "<body>%(code)d: %(message)s</body></html>" % {
            "code": status_code,
            "message": httplib.responses[status_code],
        }



    def async_callback(self, callback, *args, **kwargs):
        """Wrap callbacks with this if they are used on asynchronous requests.

        Catches exceptions and properly finishes the request.
        """
        if callback is None:
            return None
        if args or kwargs:
            callback = functools.partial(callback, *args, **kwargs)
        def wrapper(*args, **kwargs):
            try:
                return callback(*args, **kwargs)
            except Exception, e:
                if self._headers_written:
                    logging.error("Exception after headers written",
                                  exc_info=True)
                else:
                    self._handle_request_exception(e)
        return wrapper


    def _execute(self, *args, **kwargs):
        try:
            self.prepare()
            if not self._finished:  
                getattr(self, self.request.method.lower())(*args, **kwargs)
                if self._auto_finish and not self._finished:
                    self.finish()
        except Exception, e:
            self._handle_request_exception(e)

    def _generate_headers(self):
        headers = self._headers
        lines = [self.request.version + " " + str(self._status_code) + " " +
                 httplib.responses[self._status_code]]
        lines.extend(["%s: %s" % (n, v) for n, v in self._headers.iteritems()])
        return "\r\n".join(lines) + "\r\n\r\n"

    def _log(self):
        if self._status_code < 400:
            log_method = logging.info
        elif self._status_code < 500:
            log_method = logging.warning
        else:
            log_method = logging.error
        request_time = 1000.0 * self.request.request_time()
        log_method("%d %s %.2fms", self._status_code,
                   self._request_summary(), request_time)

    def _request_summary(self):
        return self.request.method + " " + self.request.uri + " (" + \
            self.request.remote_ip + ")"

    def _handle_request_exception(self, e):
        logging.error("Uncaught exception %s\n%r", self._request_summary(),
                      self.request, exc_info=e)
        self.send_error(500)


def asynchronous(method):
    """Wrap request handler methods with this if they are asynchronous.

    If this decorator is given, the response is not finished when the
    method returns. It is up to the request handler to call self.finish()
    to finish the HTTP request. Without this decorator, the request is
    automatically finished when the get() or post() method returns.

       class MyRequestHandler(web.RequestHandler):
           @web.asynchronous
           def get(self):
              http = httpclient.AsyncHTTPClient()
              http.fetch("http://friendfeed.com/", self._on_download)

           def _on_download(self, response):
              self.write("Downloaded!")
              self.finish()

    """
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        if self.application._wsgi:
            raise Exception("@asynchronous is not supported for WSGI apps")
        self._auto_finish = False
        return method(self, *args, **kwargs)
    return wrapper



class Application(object):
    """A collection of request handlers that make up a web application.

    Instances of this class are callable and can be passed directly to
    HTTPServer to serve the application:

        application = web.Application([
            (r"/", MainPageHandler),
        ])
        http_server = httpserver.HTTPServer(application)
        http_server.listen(8080)
        ioloop.IOLoop.instance().start()

    """
    def __init__(self, handler):
        self.handlers = []
        self.handler = handler

    
    def __call__(self, request):
        """Called by HTTPServer to execute the request."""
        handler_class, app = self.handler
        handler = handler_class(self,request,app)
        handler._execute()
        return handler





def start():
    try:
        import simple_server
    except:
        import traceback
        traceback.print_exc()
        reload(simple_server)
    else:
        try:
            simple_server.run(9889)
        except:
            import traceback
            traceback.print_exc()
        

from time import sleep
from mypy.reload_server import auto_reload
import sys
while 1:
    auto_reload(start)
    print "\nSleep 4 seconds"
    for i in xrange(10,0,-1):
        sleep(0.4)
        print i,
    print ""



#coding:utf-8

from types import FunctionType
from byteplay import Code, opmap

from render import Render,render_func,render_doc,get_request
from urlroute import Route
from decorator import decorator

_ROUTE = Route()

def _transmute(opcode, arg):
    if (
        (opcode == opmap['LOAD_GLOBAL'])
        and
        arg == 'request'
    ):
        return opmap['LOAD_FAST'], arg
    return opcode, arg

def _render_para_func(func):
    code = Code.from_code(func.func_code)
    code.args = tuple(['request'] + list(code.args))
    code.code = [_transmute(op, arg) for op, arg in code.code]
    func.func_code = code.to_code()
    return func

class RequestRoute(object):
    def __call__(self,func):
        func = _ROUTE(func)
        return wrap_func(func)
        
    def __getattr__(self,name):
        attr = getattr(_ROUTE,name)
        def tmp(func):
            func = wrap_func(func)
            func = attr(func)
            return func
        return tmp

route = RequestRoute()

def  _wrap_func(func,*args, **kwds):
    r = func(get_request(),*args, **kwds)
    return r

def wrap_func(func):
    f = decorator(_wrap_func, func)
    func = _render_para_func(func)
    return f

def route_render_func(func):
    return _ROUTE(render_func(func))

def route_render_doc(func):
    return _ROUTE(route_render_doc(func))
    



#coding:utf-8
from myconf import config

from os.path import join
from mypy import mypyconfig
#模板渲染的查找函数
mypyconfig.LOOKUP=dict(
    directories=join(config.PREFIX,'mysite/htm'),
    module_directory=join(config.PREFIX,'tmp/htm'),
    disable_unicode=True,
    encoding_errors="ignore",
    default_filters=['str', 'h'],
    filesystem_checks=config.MAKO_FILESYSTEM_CHECK
)
mypyconfig.FUNC_MODULE_PREFIX_LEN=len("mysite.ctrl.")


if config.THREAD_SAFE:
    import threading
    mypyconfig.LOCAL = threading.local()

from mypy.route_render import _ROUTE
from mypy.http_exc import Http404
from mypy import render
import mysite.ctrl

#初始化url route
_ROUTE._install(mysite.ctrl,"mysite.ctrl")

def application(request):
    render.LOCAL.request = request
    uri = request.uri
    try:
        result = _ROUTE << uri.path
    except Http404:
        result = "HTTPNotFound: %s"%(uri.path)
    except Exception:
        import traceback
        traceback.print_exc()
        raise
    return str(result)

from mypy.profile_middleware import ProfileMiddleware
application = ProfileMiddleware(application)


from mypy.gzip import GzipMiddleware
application = GzipMiddleware(application)

from mypy.yaro import Yaro, Response
application = Yaro(application)

try:
    if config.DEBUG:
        from weberror.evalexception import EvalException
        application = EvalException(application,)
    else:
        from weberror.errormiddleware import ErrorMiddleware
        application = ErrorMiddleware(application,debug=True)
except ImportError:
    import traceback
    traceback.print_exc()

from mysite.model import init_db




def iunique(iterable):
    seen = set()
    for i in iterable:
        if i not in seen:
            seen.add(i)
            yield i

def unique(iterable):return list(iunique(iterable))

def inplace_unique_extend(*args):
    first = args[0]
    seen = set(first)
    for iterable in args[1:]:
        for i in iterable:
            if i not in seen:
                seen.add(i)
                first.append(i)
    return first


#coding:utf-8
class Jsdict(object):
    def __init__(self,d=None):
        if d is None:
            d = {}
        self.__dict__["_Jsdict__d"] = d

    def __iter__(self):
        for i in self.__d.iteritems():
            yield i

    def __nonzero__(self):
        return bool(self.__d)

    def __setattr__(self,name,val):
        if val is not None:
            self.__d[name] = val

    def __getattr__(self,name):
        return self.__d.get(name,"")

    def __getitem__(self,name):
        return self.__d.get(name,"")

    def __contains__(self,b):
        return b in self.__d

    def __setitem__(self,name,val):
        if val is not None:
            self.__d[name] = val
        
    def __delitem__(self,name):
        del self.__d[name]

WHITE_SPACE = "\r\n \tĦĦ"
class StripJsdict(object):
    def __init__(self,form):
        self.__dict__["_StripJsdict__f"] = form
    
    def __getattr__(self,name):
        return self.__f[name].strip(WHITE_SPACE)
        
    def __getitem__(self,name):
        return self.__f[name].strip(WHITE_SPACE)

    def __iter__(self):
        for i in self.__f:
            yield i

    def __nonzero__(self):
        return bool(self.__f)

    def __setattr__(self,name,val):
        if val is not None:
            self.__f[name] = val

    def __contains__(self,b):
        return b in self.__f

    def __setitem__(self,name,val):
        if val is not None:
            self.__f[name] = val
        
    def __delitem__(self,name):
        del self.__f[name]



#!/usr/bin/env python
#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""A non-blocking, single-threaded HTTP server."""

import cgi
import errno
import fcntl
import functools
import ioloop
import iostream
import logging
import socket
import time
import urlparse


class HTTPServer(object):
    """A non-blocking, single-threaded HTTP server.

    A server is defined by a request callback that takes an HTTPRequest
    instance as an argument and writes a valid HTTP response with
    request.write(). request.finish() finishes the request (but does not
    necessarily close the connection in the case of HTTP/1.1 keep-alive
    requests). A simple example server that echoes back the URI you
    requested:

        import httpserver
        import ioloop

        def handle_request(request):
           message = "You requested %s\n" % request.uri
           request.write("HTTP/1.1 200 OK\r\nContent-Length: %d\r\n\r\n%s" % (
                         len(message), message))
           request.finish()

        http_server = httpserver.HTTPServer(handle_request)
        http_server.listen(8888)
        ioloop.IOLoop.instance().start()

    HTTPServer is a very basic connection handler. Beyond parsing the
    HTTP request body and headers, the only HTTP semantics implemented
    in HTTPServer is HTTP/1.1 keep-alive connections. We do not, however,
    implement chunked encoding, so the request callback must provide a
    Content-Length header or implement chunked encoding for HTTP/1.1
    requests for the server to run correctly for HTTP/1.1 clients. If
    the request handler is unable to do this, you can provide the
    no_keep_alive argument to the HTTPServer constructor, which will
    ensure the connection is closed on every request no matter what HTTP
    version the client is using.

    If xheaders is True, we support the X-Real-Ip and X-Scheme headers,
    which override the remote IP and HTTP scheme for all requests. These
    headers are useful when running Tornado behind a reverse proxy or
    load balancer.
    """
    def __init__(self, request_callback, no_keep_alive=False, io_loop=None,
                 xheaders=False):
        self.request_callback = request_callback
        self.no_keep_alive = no_keep_alive
        self.io_loop = io_loop or ioloop.IOLoop.instance()
        self.xheaders = xheaders
        self._socket = None

    def listen(self, port):
        assert not self._socket
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        flags = fcntl.fcntl(self._socket.fileno(), fcntl.F_GETFD)
        flags |= fcntl.FD_CLOEXEC
        fcntl.fcntl(self._socket.fileno(), fcntl.F_SETFD, flags)
        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._socket.setblocking(0)
        self._socket.bind(("", port))
        self._socket.listen(128)
        self.io_loop.add_handler(self._socket.fileno(), self._handle_events,
                                 self.io_loop.READ)

    def _handle_events(self, fd, events):
        while True:
            try:
                connection, address = self._socket.accept()
            except socket.error, e:
                if e[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
                    return
                raise
            try:
                stream = iostream.IOStream(connection, io_loop=self.io_loop)
                HTTPConnection(stream, address, self.request_callback,
                               self.no_keep_alive, self.xheaders)
            except:
                logging.error("Error in connection callback", exc_info=True)


class HTTPConnection(object):
    """Handles a connection to an HTTP client, executing HTTP requests.

    We parse HTTP headers and bodies, and execute the request callback
    until the HTTP conection is closed.
    """
    def __init__(self, stream, address, request_callback, no_keep_alive=False,
                 xheaders=False):
        self.stream = stream
        self.address = address
        self.request_callback = request_callback
        self.no_keep_alive = no_keep_alive
        self.xheaders = xheaders
        self._request = None
        self._request_finished = False
        self.stream.read_until("\r\n\r\n", self._on_headers)

    def write(self, chunk):
        assert self._request, "Request closed"
        self.stream.write(chunk, self._on_write_complete)

    def finish(self):
        assert self._request, "Request closed"
        self._request_finished = True
        if not self.stream.writing():
            self._finish_request()

    def _on_write_complete(self):
        if self._request_finished:
            self._finish_request()

    def _finish_request(self):
        if self.no_keep_alive:
            disconnect = True
        else:
            connection_header = self._request.headers.get("Connection")
            if self._request.supports_http_1_1():
                disconnect = connection_header == "close"
            elif ("Content-Length" in self._request.headers 
                    or self._request.method in ("HEAD", "GET")):
                disconnect = connection_header != "Keep-Alive"
            else:
                disconnect = True
        self._request = None
        self._request_finished = False
        if disconnect:
            self.stream.close()
            return
        self.stream.read_until("\r\n\r\n", self._on_headers)

    def _on_headers(self, data):
        eol = data.find("\r\n")
        start_line = data[:eol]
        method, uri, version = start_line.split(" ")
        if not version.startswith("HTTP/"):
            raise Exception("Malformed HTTP version in HTTP Request-Line")
        headers = HTTPHeaders.parse(data[eol:])
        self._request = HTTPRequest(
            connection=self, method=method, uri=uri, version=version,
            headers=headers, remote_ip=self.address[0])

        content_length = headers.get("Content-Length")
        if content_length:
            content_length = int(content_length)
            if content_length > self.stream.max_buffer_size:
                raise Exception("Content-Length too long")
            if headers.get("Expect") == "100-continue":
                self.stream.write("HTTP/1.1 100 (Continue)\r\n\r\n")
            self.stream.read_bytes(content_length, self._on_request_body)
            return

        self.request_callback(self._request)

    def _on_request_body(self, data):
        self._request.body = data
        content_type = self._request.headers.get("Content-Type", "")
        if self._request.method == "POST":
            if content_type.startswith("application/x-www-form-urlencoded"):
                arguments = cgi.parse_qs(self._request.body)
                for name, values in arguments.iteritems():
                    values = [v for v in values if v]
                    if values:
                        self._request.arguments.setdefault(name, []).extend(
                            values)
            elif content_type.startswith("multipart/form-data"):
                boundary = content_type[30:]
                if boundary: self._parse_mime_body(boundary, data)
        self.request_callback(self._request)

    def _parse_mime_body(self, boundary, data):
        if data.endswith("\r\n"):
            footer_length = len(boundary) + 6
        else:
            footer_length = len(boundary) + 4
        parts = data[:-footer_length].split("--" + boundary + "\r\n")
        for part in parts:
            if not part: continue
            eoh = part.find("\r\n\r\n")
            if eoh == -1:
                logging.warning("multipart/form-data missing headers")
                continue
            headers = HTTPHeaders.parse(part[:eoh])
            name_header = headers.get("Content-Disposition", "")
            if not name_header.startswith("form-data;") or \
               not part.endswith("\r\n"):
                logging.warning("Invalid multipart/form-data")
                continue
            value = part[eoh + 4:-2]
            name_values = {}
            for name_part in name_header[10:].split(";"):
                name, name_value = name_part.strip().split("=", 1)
                name_values[name] = name_value.strip('"').decode("utf-8")
            if not name_values.get("name"):
                logging.warning("multipart/form-data value missing name")
                continue
            name = name_values["name"]
            if name_values.get("filename"):
                ctype = headers.get("Content-Type", "application/unknown")
                self._request.files.setdefault(name, []).append(dict(
                    filename=name_values["filename"], body=value,
                    content_type=ctype))
            else:
                self._request.arguments.setdefault(name, []).append(value)


class HTTPRequest(object):
    """A single HTTP request.

    GET/POST arguments are available in the arguments property, which
    maps arguments names to lists of values (to support multiple values
    for individual names). Names and values are both unicode always.

    File uploads are available in the files property, which maps file
    names to list of files. Each file is a dictionary of the form
    {"filename":..., "content_type":..., "body":...}. The content_type
    comes from the provided HTTP header and should not be trusted
    outright given that it can be easily forged.

    An HTTP request is attached to a single HTTP connection, which can
    be accessed through the "connection" attribute. Since connections
    are typically kept open in HTTP/1.1, multiple requests can be handled
    sequentially on a single connection.
    """
    def __init__(self, method, uri, version="HTTP/1.0", headers=None,
                 body=None, remote_ip=None, protocol=None, host=None,
                 files=None, connection=None):
        self.method = method
        self.uri = uri
        self.version = version
        self.headers = headers or HTTPHeaders()
        self.body = body or ""
        if connection and connection.xheaders:
            self.remote_ip = headers.get("X-Real-Ip", remote_ip)
            self.protocol = headers.get("X-Scheme", protocol) or "http"
        else:
            self.remote_ip = remote_ip
            self.protocol = protocol or "http"
        self.host = host or headers.get("Host") or "127.0.0.1"
        self.files = files or {}
        self.connection = connection
        self._start_time = time.time()
        self._finish_time = None

        scheme, netloc, path, query, fragment = urlparse.urlsplit(uri)
        self.path = path
        self.query = query
        arguments = cgi.parse_qs(query)
        self.arguments = {}
        for name, values in arguments.iteritems():
            values = [v for v in values if v]
            if values: self.arguments[name] = values

    def supports_http_1_1(self):
        """Returns True if this request supports HTTP/1.1 semantics"""
        return self.version == "HTTP/1.1"

    def write(self, chunk):
        """Writes the given chunk to the response stream."""
        assert isinstance(chunk, str)
        self.connection.write(chunk)

    def finish(self):
        """Finishes this HTTP request on the open connection."""
        self.connection.finish()
        self._finish_time = time.time()

    def full_url(self):
        """Reconstructs the full URL for this request."""
        return self.protocol + "://" + self.host + self.uri

    def request_time(self):
        """Returns the amount of time it took for this request to execute."""
        if self._finish_time is None:
            return time.time() - self._start_time
        else:
            return self._finish_time - self._start_time

    def __repr__(self):
        attrs = ("protocol", "host", "method", "uri", "version", "remote_ip",
                 "remote_ip", "body")
        args = ", ".join(["%s=%r" % (n, getattr(self, n)) for n in attrs])
        return "%s(%s, headers=%s)" % (
            self.__class__.__name__, args, dict(self.headers))


class HTTPHeaders(dict):
    """A dictionary that maintains Http-Header-Case for all keys."""
    def __setitem__(self, name, value):
        dict.__setitem__(self, self._normalize_name(name), value)

    def __getitem__(self, name):
        return dict.__getitem__(self, self._normalize_name(name))

    def _normalize_name(self, name):
        return "-".join([w.capitalize() for w in name.split("-")])

    @classmethod
    def parse(cls, headers_string):
        headers = cls()
        for line in headers_string.splitlines():
            if line:
                name, value = line.split(": ", 1)
                headers[name] = value
        return headers



from setuptools import setup, find_packages

version = '0.231'

setup(
    name="mypy",
    version=version,
    description="A wsgi framework",
    long_description=""" """,
    author="zsp",
    include_package_data=True,
    zip_safe=False,
    author_email="zsp007@gmail.com",
    packages=find_packages(),
    install_requires = ['decorator','hmako>=1.12','mako','sqlbean>=0.44','weberror'],
)



PROFILE_FUNC_LIST = []
from mypy import cookie

import hotshot,hotshot.stats
from os import getpid
from cStringIO import StringIO
import os
import re
import time
from os.path import abspath,dirname,join
from tempfile import gettempdir 
tempdir=gettempdir()
PROFILE_LOG_FILENAME_PATTERN = join(tempdir,'profile-%s.prof')
PROFILE_PICKLE_FILENAME_PATTERN = join(tempdir,'profile-%s.pickle')
PROFILE_LIMIT = 60

class ProfileInfo:
    """
    Stage 1: profiling
    Stage 2: profile stop, waiting for check result
    """
    def __init__(self, stage, pid, profiler_id=None):
        self.stage = stage
        self.pid = pid
        self.profiler_id = profiler_id and str(profiler_id)

    def get_stats(self):
        stats = load_stats(PROFILE_LOG_FILENAME_PATTERN % self.pid)
        s1 = format_stats(stats, sort='time')
        s2 = format_stats(stats, sort='cumulative')
        return s1 + '\n'*3 + '='*80 + '\n'*3 + s2

def request_start_profile(request, profiler_id, req_count=100):
    request.start_profile = req_count
    pid = getpid()
    info = ProfileInfo(1, pid, profiler_id)
    pickle_fn = PROFILE_PICKLE_FILENAME_PATTERN % pid
    dump(info, open(pickle_fn, 'wb'))

def start_profile(filename=None):
    if filename is None:
        filename = PROFILE_LOG_FILENAME_PATTERN % getpid()
    prof = hotshot.Profile(filename)
    return prof

def on_profile_finished():
    pid = getpid()
    pickle_fn = PROFILE_PICKLE_FILENAME_PATTERN % pid
    try:
        info = load(open(pickle_fn, 'rb'))
        info.stage = 2
    except IOError:
        info = ProfileInfo(2, pid)
    dump(info, open(pickle_fn, 'wb'))

def get_profile_info(pid=None):
    pid = pid or os.getpid()
    pickle_fn = PROFILE_PICKLE_FILENAME_PATTERN % pid
    try:
        profinfo = load(open(pickle_fn, 'rb'))
        return profinfo
    except IOError:
        return None

def delete_profile_info(pid=None):
    pid = pid or os.getpid()
    try:
        os.remove(PROFILE_PICKLE_FILENAME_PATTERN % pid)
    except IOError:
        pass

def load_stats(filename=None):
    if filename is None:
        filename = PROFILE_LOG_FILENAME_PATTERN % getpid()
    stats = hotshot.stats.load(filename)
    try:
        os.remove(filename)
    except IOError:
        pass
    return stats

def format_stats(stats, strip_dirs=None, sort='time'):
    if strip_dirs:
        stats.strip_dirs()
    stats.sort_stats(sort, 'calls')
    so = StringIO()
    stats.stream = so
    stats.print_stats(PROFILE_LIMIT)
    stats.print_callers(PROFILE_LIMIT)
    r = so.getvalue()
    if not strip_dirs:
        rootdir = dirname(
            dirname(dirname(dirname(abspath(__file__))))
        ).lower()+os.sep
        r = r.replace(rootdir,"")
    return r

normalize_re = re.compile(r'\d+')
class CallLogger(object):
    """Log every call"""

    def __init__(self, obj):
        self.obj = obj
        self.log = []

    def __getattr__(self, name):
        attr = getattr(self.obj, name)
        def _(*a, **kw):
            call = format_call(name, *a, **kw)
            ncall = normalize_re.sub('-', call)
            t1 = time.time()
            r = attr(*a, **kw)
            cost = time.time() - t1
            self.log.append((call, ncall, cost))
            return r
        return _

def format_call(funcname, *a, **kw):
    arglist = []
    for v in a:
        v = repr(v)
        if len(v)>70:
            v=v[:70]+"..."
        arglist.append(v)

    r = []
    for k, v in kw.iteritems():
        if len(v)>70:
            v=v[:70]+"..."
        r.append((k,v))
        
    arglist += ["%s=%r" % (k,v) for k,v in r]
    return "%s(%s)" % (funcname, ", ".join(arglist))

def ProfileMiddleware(application):
    def _application(request):
        profile = request.query.profile
        cookie_profile = request.cookie.get("profile")
        if profile or cookie_profile:
            if profile=="0":
                cookie.delete("profile",request=request)
            else:
                if not cookie_profile:
                    cookie.set("profile","1",request=request)
                #from zku.autumn.model import sqlstore
                prof = start_profile()
                for i in PROFILE_FUNC_LIST:
                    i.start_log()
                #sqlstore.
                #mc.start_log()
                so = []
                result = None
                try:
                    result = prof.runcall(application,request)
                    for i in PROFILE_FUNC_LIST:
                        so.append(i.get_log())
                #    so += sqlstore.get_log()
                #    so += mc.get_log()
                finally:
                    for i in PROFILE_FUNC_LIST:
                        i.stop_log()
                #    mc.stop_log()
                #    sqlstore.stop_log()
                    prof.close()

                if result is not None and (result.startswith("<") or profile):
                    #disable output for json
                    stats = load_stats()
                    
                    so.append( format_stats(stats, sort='time') )
                    so.append( format_stats(stats, sort='cumulative') )
                    
                    from cgi import escape
                    splitor = "\n"+"=*"*60+"\n\n\n"
                    so = escape(splitor.join(so))
                    so = """<pre style="clear:both;padding:1em;background:#ffffde;color:#033">\n%s\n</pre>"""%so
                    result = "%s %s"%(result,so)
                return result
        return application(request)
    return _application



#coding:utf-8
from init_db import McCache

from email.MIMEText import MIMEText
from email.Header import Header
from email.Utils import parseaddr, formataddr
from base64 import encodestring

NOT_SUPPORT_UTF8_DOMAIN = set(['tom.com', 'hotmail.com', 'msn.com', 'yahoo.com'])

def ignore_encode(s, enc):
    return s.decode('utf-8', 'ignore').encode(enc, 'ignore')

import  smtplib
from myconf.config import PREFIX,SMTP,SMTP_USERNAME,SMTP_PASSWORD,SYS_EMAIL_SENDER,SYS_EMAIL_SENDER_NAME

def send_email_imp(
                smtp,
                sender, sender_name,
                recipient, recipient_name,
                subject, body, enc='utf-8',
                format='plain'
            ):
    
    if not subject:
        return
    
    at = recipient.find('@')
    if at <= 0:
        return
    
    domain = recipient[at+1:].strip()
    if domain not in NOT_SUPPORT_UTF8_DOMAIN:
        enc = 'utf-8'
    else:
        enc = "gb18030"

    if enc.lower() != 'utf-8':
        sender_name = ignore_encode(sender_name)
        recipient_name = ignore_encode(recipient_name)
        body = ignore_encode(body)
        subject = ignore_encode(subject, enc)

    msg = MIMEText(body, format, enc)
    msg['Subject'] = Header(subject, enc)
    
    sender_name = str(Header(sender_name, enc))
    msg['From'] = formataddr((sender_name, sender))

    recipient_name = str(Header(recipient_name, enc))
    msg['To'] = formataddr((recipient_name, recipient))

    smtp.sendmail(sender, recipient, msg.as_string())


from os.path import join
from decorator import decorator
from mypy.byteplay import Code, opmap


TXT_PATH = join(PREFIX,"mysite/txt")

McMailTmp = McCache("MailTmp:%s")

def render_template(uri,**kwds):
    txt = McMailTmp.get(uri)
        
    if txt is None:
        with open("%s/%s.txt"%(TXT_PATH,uri)) as template:
            txt = template.read()
            McMailTmp.set(uri,txt,600)
    
    txt = txt.format(**kwds)
    r = txt.split("\n",1)
    if len(r)<2:
        r.append(txt[0])
    return r

def send_email(subject,text,email,name,sender=SYS_EMAIL_SENDER,sender_name=SYS_EMAIL_SENDER_NAME):
    server = smtplib.SMTP(SMTP)
    server.login(SMTP_USERNAME,SMTP_PASSWORD)

    send_email_imp(server,sender,sender_name,email,name,subject,text)

    server.quit()

def render_email(uri,email,name,sender=SYS_EMAIL_SENDER,sender_name=SYS_EMAIL_SENDER_NAME,**kwds):
    if "name" not in kwds:
        kwds['name'] = name
    
    if "email" not in kwds:
        kwds['email'] = email
    
    if "sender" not in kwds:
        kwds['sender'] = sender
    
    if "sender_name" not in kwds:
        kwds['sender_name'] = sender_name
    
    subject,text = render_template(uri,**kwds)
    send_email(subject,text,email,name,sender,sender_name)



import sys,os,subprocess

def quote_first_command_arg( arg): 
    """ 
    There's a bug in Windows when running an executable that's 
    located inside a path with a space in it.  This method handles 
    that case, or on non-Windows systems or an executable with no 
    spaces, it just leaves well enough alone. 
    """ 
    if (sys.platform != 'win32' 
        or ' ' not in arg): 
        # Problem does not apply: 
        return arg 
    try: 
        import win32api 
    except ImportError: 
        raise ValueError( 
            "The executable %r contains a space, and in order to " 
            "handle this issue you must have the win32api module " 
            "installed" % arg) 
    arg = win32api.GetShortPathName(arg) 
    return arg 

def _turn_sigterm_into_systemexit():
    """
    Attempts to turn a SIGTERM exception into a SystemExit exception.
    """
    try:
        import signal
    except ImportError:
        return
    def handle_term(signo, frame):
        raise SystemExit
    signal.signal(signal.SIGTERM, handle_term)

def restart_with_monitor():
    while 1:
        args = [quote_first_command_arg(sys.executable)] + sys.argv
        new_environ = os.environ.copy()
        new_environ['_run']="1"
        proc = None
        try:
            try:
                _turn_sigterm_into_systemexit()
                proc = subprocess.Popen(args, env=new_environ)
                exit_code = proc.wait()
                proc = None
            except KeyboardInterrupt:
                print '^C caught in monitor process'
                return 1
        finally:
            if (proc is not None
                and hasattr(os, 'kill')):
                import signal
                try:
                    os.kill(proc.pid, signal.SIGTERM)
                except (OSError, IOError):
                    pass
            
        # Reloader always exits with code 3; but if we are
        # a monitor, any exit code will restart
        if exit_code != 3:
                return exit_code

def auto_reload(app):
    if os.environ.get("_run"):
        import reloader
        reloader.install()
        app()
    else:
        restart_with_monitor()

